<?php

namespace Drupal\chatgpt\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;

/**
 * Function to create the ChatGPT Search popup.
 */
class ChatGPTForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'chatgpt_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['chatgpt_search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search content in ChatGPT'),
      '#required' => TRUE,
    ];

    $form['chatgpt_submit'] = [
      '#type' => 'button',
      '#value' => 'Search',
      '#ajax' => [
        'callback' => '::chatgptSearchResult',
        'effect' => 'fade',
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Getting Result from OpenAI ChatGPT...'),
        ],
      ],
    ];

    $form['chatgpt_result'] = [
      '#type' => 'markup',
      '#title' => $this->t('ChatGPT Result'),
      '#markup' => '<div id="chatgpt-result"></div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * Function to call the Chat GPT API and get the result.
   */
  public function chatgptSearchResult(array &$form, FormStateInterface $form_state) {
    $ajax_response = new AjaxResponse();
    $config = $this->config('chatgpt.adminsettings');
    $prompt_text = $form_state->getValue('chatgpt_search');
    $url = $config->get('chatgpt_endpoint');
    $model = $config->get('chatgpt_model');
    $access_token = $config->get('chatgpt_token');
    $temperature = $config->get('chatgpt_temperature');
    $max_tokens = $config->get('chatgpt_max_token');

    $payload = [
      "model" => $model,
      "prompt" => $prompt_text,
      "temperature" => (int) $temperature,
      "max_tokens" => (int) $max_tokens,
    ];
    $json_input = json_encode($payload);
    $headr = [];
    $headr[] = 'Content-Type: application/json';
    $headr[] = 'Authorization: Bearer ' . $access_token;

    // cURL starts.
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_POST, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $json_input);
    curl_setopt($curl, CURLOPT_HTTPHEADER, $headr);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    $reply = curl_exec($curl);
    $http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    // cURL ends
    $decoded_data = json_decode($reply);

    // Error handling for OpenAI API.
    if ($http_status != 200) {
      $error_msg = $decoded_data->error->type.": ".$decoded_data->error->message;
      $ajax_response->addCommand(new HtmlCommand('#chatgpt-result', $error_msg));
      return $ajax_response;
    }
    else {
      // processing success response data.
      $text = $decoded_data->choices[0]->text;
      $ajax_response->addCommand(new HtmlCommand('#chatgpt-result', $text));
      return $ajax_response;
    }

  }

}
